'use strict'

const co = require('co');
const cli = require('heroku-cli-util');
const app = 'cake-data';
const downloadBackup = require('heroku-pg/commands/backups/download');
const exec = require('child_process').exec;
const RSVP = require('rsvp');

const runCmd = function(cmd) {
  return new RSVP.Promise(function(resolve, reject) {
    exec(cmd, function(error, stdout, stderr) {
      if (error) {
        cli.error(error);
        reject();
      }

      resolve();
    });
  });
};

const closeConnections = function(localDb) {
  return runCmd(`psql -d  ${localDb} -c 'SELECT pg_terminate_backend(pid) FROM pg_stat_activity WHERE pid <> pg_backend_pid();'`);
};

const dropAndCreate = function() {
  return runCmd('RAILS_ENV=development bundle exec rake db:drop && bundle exec rake db:create');
};

const restore = function(localDb) {
  return runCmd(`pg_restore --no-acl --no-owner -d ${localDb} latest.dump`);
};

const deleteLocalBackup = function() {
  return runCmd('rm -f latest.dump');
};

const envVariablesMissing  = function(localDb, remoteApp) {
  let variablesMissing = false;

  if (!localDb) {
    cli.error('You need to set CAKE_LOCAL_DB as an environment variable');
    variablesMissing = true;
  }

  if (!remoteApp) {
    cli.error('You need to set CAKE_REMOTE_APP as an environment variable');
    variablesMissing = true;
  }

  return variablesMissing;
}

function* fetchBackupUrl (context, heroku) {
  let localDb = process.env.CAKE_LOCAL_DB;
  let remoteApp = process.env.CAKE_REMOTE_APP;

  if (envVariablesMissing(localDb, remoteApp)) {
    return;
  }

  context.app = remoteApp;

  yield downloadBackup.run(context, heroku);

  yield cli.action('Restoring local database', co(function* () {
    yield closeConnections(localDb);
    yield dropAndCreate();
    yield restore(localDb);
    yield deleteLocalBackup();
  }));
}

module.exports = {
  topic: 'cake',
  command: 'slice',
  description: 'pulls a backup of the db and populates a local db',
  help: '',
  needsAuth: true,
  run: cli.command(co.wrap(fetchBackupUrl))
}
